////////////////////////////////////////////////////////////////////////////
// Special types and constants for sx17.                                  //
////////////////////////////////////////////////////////////////////////////

typedef unsigned long		DWORD;
typedef long				LONG;
typedef unsigned short		WORD;
typedef unsigned char		BYTE;

#define	SYS_BUFS			8				// internal capture buffers

//	MODE structure
typedef struct {
	unsigned long	format;                 //  NTSC, PAL
	unsigned long   bmpflip;				//  bmp capture in Windows or natural lines order
	unsigned long   bmpcolor;				//  bmp color or monochrome
	unsigned long   bmpsize;                //  bmp large (256x192), small (128x96), huge (640x240)
	unsigned long   jpgsize;				//  JPEG size: 640x480, 320x240, 640x240
	unsigned long   captionformat;			//  format of the caption window
	unsigned long   captionpos;				//  position of the caption window
	unsigned long   fontsize;				//  caption font size
    unsigned long   diomask;                //  digital i/o mask (a 0 enables corresponding bit as output)
    unsigned long   dioinit;                //  digital i/o initial setting (done by SetMode)
    unsigned long   boardver;               //  board version (16 or 4-input)
} MODE;

//	BUFFER structure
typedef struct {
	int				bi;						//	buffer index
    int             valid;                  //  1 if valid data captured, 0 otherwise
	unsigned long   jpegsize;               //	actual size of jpeg data;
	int             xpchan;                 //	one of 16 crosspoint switch channels (1-16) captured from;
    int             vxp;                    //  current state of video out crosspoint switch
	int             videopresent;           //	video present flag;
    unsigned long   dioin;                  //  digital i/o read (in) data
	int             compfactor;				//  compression factor for captured jpeg
    int             bright;                 //  brightness setting for captured image
    int             contrast;               //  contrast setting for captured image
    int             sat;                    //  saturation setting for captured image
    int             hue;                    //  hue setting for captured image
	int				incomplete;				//	capture incomplete flag
	int				zerr;					//	compression error flag
    int             reserved[8];
} BUFFER, *PBUFFER;

//	COMMAND structure
typedef struct {
	int             jpegon;                 //	JPEG capture ON/OFF
    int             bmpon;                  //  BMP capture on/off
	int             xpchan;                 //	crosspoint switch input to switch to next (1-16 or 1-4 for a 4-input board);
	int             compfactor;				//  compression factor
    int             bright;                 //  brightness: normal: 0x80, max: 0xff, min: 0x00
    int             contrast;               //  contrast: normal: 0x44, max: 0x7f, min: 0x00
    int             sat;                    //  saturation: normal 0x40, max: 0x7f, min: 0x00
    int             hue;                    //  hue: normal: 0x00, max positive: 0x7f, max negative: 0x80
	int             captionmode;            //	caption mode for current capture: ON/OFF, caption color;
	void            *pCaption;              //	pointer to caption data buffer (64 bytes);
	int             offset;                 //	channel number offset (for ^c caption formatting function)
    int             vxp;                    //  video out crosspoint switch control word;
    int             dioout;                 //  digital i/o write (out) data;
} COMMAND, *PCOMMAND;

// OFFSET struct: gives offset of the specific buffer into mmap'ed area
typedef struct {
	unsigned long	obmp;					//bmp data offset
	unsigned long	ojpg;					//jpg data offset
} OFFSET;


///////////////////////////////////////////////////////////////////////////////////////
// ioctl constants
#define SX17_IOC_SETMODE		0x2000				// set operating mode.
#define SX17_IOC_GETOFFSETS		0x2010				// get buffer offsets
#define SX17_IOC_QUEUECMD		0x2020				// queue capture command
#define SX17_IOC_GETBUFR		0x2030				// get BUFFER struct
#define SX17_IOC_RELEASEBUFR	0x2040				// release buffer
#define SX17_IOC_GETMAPSIZE		0x2050				// get mmap'ed buffer size

//MODE constants
#define	    FORMAT_NTSC         0
#define	    FORMAT_PAL          1

#define	    FLIP_OFF            0
#define     FLIP_ON             1           //image is stored upside down so that Windows displays it correctly

#define     COLOR_MONO          0           //momochrome bitmap (1 byte/pixel)
#define	    COLOR_YCRCB         1	    	//color bitmap (YCrCb format, 2 bytes/pixel, CbYCrY)
#define	    COLOR_RGB           2	    	//color bitmap (RGB format, 3 bytes/pixel)
#define	    COLOR_YCRCB_BS		4		    //color bitmap (YCrCb format, 2 bytes/pixel, YCbYCr)

#define     BMP_SIZE_LRG        0		    //large bitmap (256x192)
#define	    BMP_SIZE_SML		1   		//small bitmap (128x96)
#define	    BMP_SIZE_HUG		2   		//huge bitmap (640x240) (allowed only with JPG_SIZE_FLD)

#define	    JPG_SIZE_FULL		1           //full size JPEG (640x480)
#define	    JPG_SIZE_HALF		0   		//half size JPEG (320x240)
#define	    JPG_SIZE_FLD		2   		//field size JPEG (640x240) (allowed only with BMP_SIZE_HUG)

#define     CAP8X8				0   		//8 chars by 8 lines caption
#define     CAP16X4				1   		//16 chars by 4 lines caption
#define	    CAP32X2				2           //32 chars by 2 lines caption

#define	    CP_UL				0   		//caption window in upper left corner
#define	    CP_UR				1   		//caption window in upper right corner
#define	    CP_LL				2   		//caption window in lower left corner
#define	    CP_LR				3   		//caption window in lower right corner

#define	    CF_SMALL			0   		//small caption font size
#define	    CF_LARGE			1   		//large caption font size

#define	    COMPFACTOR_MIN		0x80    	//minimum compression factor

#define     DIO_ALLINPUTS       0xffffffff  //all dio pins are inputs

//Board type: use BOARD_S for 16-input board, BOARD_S4 - for 4-input board
#define     BOARD_S             1
#define     BOARD_S4            3

#define     DEF_MODE			{FORMAT_NTSC, FLIP_ON, COLOR_RGB, BMP_SIZE_SML, JPG_SIZE_FULL, \
								 CAP16X4, CP_LL, CF_SMALL, DIO_ALLINPUTS, 0, BOARD_S}

//COMMAND constants
#define	    JPEG_ON				1   		//JPEG capture on
#define	    JPEG_OFF			0   		//JPEG capture off
#define	    BMP_ON				1   		//BMP capture on
#define	    BMP_OFF 			0   		//BMP capture off
#define	    CAP_ON				1   		//caption on
#define	    CAP_OFF				0   		//caption off
//caption color constants
#define     CAP_BLK				0x00	    //black
#define     CAP_WHT				0x10	    //white
#define     CAP_RED				0x20    	//red
#define	    CAP_BLU				0x30	    //blue
#define     CAP_GRN				0x40	    //green
#define	    CAP_YLW				0x50	    //yellow
#define     CAP_MGT				0x60	    //magenta
#define	    CAP_CYN				0x70	    //cyan

//Bitmap size constants
#define		BMPLINES_NTSC_SML	96					   	    //lines per field, NTSC
#define		BMPLINES_NTSC_LRG	BMPLINES_NTSC_SML * 2	    //lines per field, NTSC
#define		BMPPIXELS_NTSC_SML	128					        //pixels per line
#define		BMPPIXELS_NTSC_LRG	BMPPIXELS_NTSC_SML * 2	    //pixels per line
#define     BMPLINES_NTSC_HUG   240
#define		BMPPIXELS_NTSC_HUG	640					        //pixels per line

#define		BMPLINES_PAL_SML	96						    //lines per field, PAL
#define		BMPLINES_PAL_LRG	BMPLINES_PAL_SML * 2	    //lines per field, PAL
#define		BMPPIXELS_PAL_SML	128						    //pixels per line
#define		BMPPIXELS_PAL_LRG	BMPPIXELS_PAL_SML * 2	    //pixels per line
#define     BMPLINES_PAL_HUG    240
#define		BMPPIXELS_PAL_HUG	640					        //pixels per line

